@@ -7,10 +7,11 @@ import glib
 import pango
 import gettext
 import os
+import subprocess
 import re
 import logging
 from tryton.config import CONFIG
-from tryton.config import TRYTON_ICON, PIXMAPS_DIR, DATA_DIR
+from tryton.config import TRYTON_ICON, PIXMAPS_DIR
 import time
 import sys
 import xmlrpclib
@@ -35,8 +36,7 @@ try:
     import ssl
 except ImportError:
     ssl = None
-import dis
-from threading import Lock, Semaphore
+from threading import Lock
 try:
     import pytz
 except ImportError:
@@ -44,6 +44,7 @@ except ImportError:
 
 from tryton.exceptions import (TrytonServerError, TrytonError,
     TrytonServerUnavailable)
+from tryton.pyson import PYSONEncoder
 
 _ = gettext.gettext
 
@@ -66,11 +67,12 @@ class TrytonIconFactory(gtk.IconFactory):
             try:
                 pixbuf = gtk.gdk.pixbuf_new_from_file(
                         os.path.join(PIXMAPS_DIR, fname).decode('utf-8'))
-            except IOError:
+            except (IOError, glib.GError):
                 continue
+            finally:
+                self._locale_icons.add(name)
             icon_set = gtk.IconSet(pixbuf)
             self.add(name, icon_set)
-            self._locale_icons.add(name)
         for name in ('ok', 'cancel'):
             icon_set = gtk.Style().lookup_icon_set('gtk-%s' % name)
             self.add('tryton-%s' % name, icon_set)
@@ -115,18 +117,97 @@ class TrytonIconFactory(gtk.IconFactory):
             fileno, path = tempfile.mkstemp()
             with os.fdopen(fileno, 'w') as svgfile:
                 svgfile.write(icon['icon'])
-            pixbuf = gtk.gdk.pixbuf_new_from_file(path.decode(
-                sys.getfilesystemencoding().encode('utf-8')))
-            os.remove(path)
+            try:
+                pixbuf = gtk.gdk.pixbuf_new_from_file(path.decode(
+                    sys.getfilesystemencoding().encode('utf-8')))
+            except glib.GError:
+                continue
+            finally:
+                os.remove(path)
+                self._tryton_icons.remove((icon['id'], icon['name']))
+                del self._name2id[icon['name']]
+                self._loaded_icons.add(icon['name'])
             iconset = gtk.IconSet(pixbuf)
             self.add(icon['name'], iconset)
-            self._tryton_icons.remove((icon['id'], icon['name']))
-            del self._name2id[icon['name']]
-            self._loaded_icons.add(icon['name'])
 
 ICONFACTORY = TrytonIconFactory()
 ICONFACTORY.add_default()
 
+
+class ModelAccess(object):
+
+    batchnum = 100
+    _access = {}
+    _models = []
+
+    def load_models(self, refresh=False):
+        if not refresh:
+            self._access.clear()
+        del self._models[:]
+
+        try:
+            self._models = rpc.execute('model', 'ir.model', 'list_models',
+                rpc.CONTEXT)
+        except TrytonServerError:
+            pass
+
+    def __getitem__(self, model):
+        if model in self._access:
+            return self._access[model]
+        if model not in self._models:
+            self.load_models(refresh=True)
+        idx = self._models.index(model)
+        to_load = slice(max(0, idx - self.batchnum // 2),
+            idx + self.batchnum // 2)
+        try:
+            access = rpc.execute('model', 'ir.model.access', 'get_access',
+                self._models[to_load], rpc.CONTEXT)
+        except TrytonServerError:
+            access = {}
+        self._access.update(access)
+        return self._access[model]
+
+MODELACCESS = ModelAccess()
+
+
+class ViewSearch(object):
+    searches = None
+
+    def load_searches(self):
+        try:
+            self.searches = rpc.execute('model', 'ir.ui.view_search',
+                'get_search', rpc.CONTEXT)
+        except TrytonServerError:
+            self.searches = {}
+
+    def __getitem__(self, model):
+        return self.searches.get(model, [])
+
+    def add(self, model, name, domain):
+        try:
+            id_, = RPCExecute('model', 'ir.ui.view_search',
+                'create', [{
+                        'model': model,
+                        'name': name,
+                        'domain': PYSONEncoder().encode(domain),
+                        }])
+        except RPCException:
+            return
+        self.searches.setdefault(model, []).append((id_, name, domain))
+
+    def remove(self, model, id_):
+        try:
+            RPCExecute('model', 'ir.ui.view_search', 'delete', [id_])
+        except RPCException:
+            return
+        for i, domain in enumerate(self.searches[model]):
+            if domain[0] == id_:
+                del self.searches[model][i]
+                break
+
+VIEW_SEARCH = ViewSearch()
+
+
 def find_in_path(name):
     if os.name == "nt":
         sep = ';'
@@ -140,12 +221,14 @@ def find_in_path(name):
             return val
     return name
 
+
 def test_server_version(host, port):
     version = rpc.server_version(host, port)
     if not version:
         return False
     return version.split('.')[:2] == VERSION.split('.')[:2]
 
+
 def refresh_dblist(host, port):
     '''
     Return the number of database available
@@ -157,6 +240,7 @@ def refresh_dblist(host, port):
         return -1
     return rpc.db_list(host, port)
 
+
 def refresh_langlist(lang_widget, host, port):
     liststore = lang_widget.get_model()
     liststore.clear()
@@ -173,17 +257,18 @@ def refresh_langlist(lang_widget, host, port):
         liststore.insert(i, (val, key))
         if key == lang:
             index = i
-        if key == 'en_US' and index < 0 :
+        if key == 'en_US' and index < 0:
             index = i
         i += 1
     lang_widget.set_active(index)
     return lang_list
 
+
 def request_server(server_widget):
     result = False
     parent = get_toplevel_window()
     dialog = gtk.Dialog(
-        title= _('Tryton Connection'),
+        title=_('Tryton Connection'),
         parent=parent,
         flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT |
             gtk.WIN_POS_CENTER_ON_PARENT |
@@ -210,7 +295,7 @@ def request_server(server_widget):
     entry_server.set_text("localhost")
     entry_server.set_activates_default(True)
     entry_server.set_width_chars(16)
-    table.attach(entry_server, 1, 2, 0, 1,yoptions=False,
+    table.attach(entry_server, 1, 2, 0, 1, yoptions=False,
         xoptions=gtk.FILL | gtk.EXPAND)
     label_port = gtk.Label(_("Port:"))
     label_port.set_alignment(1, 0.5)
@@ -241,15 +326,17 @@ def request_server(server_widget):
     dialog.destroy()
     return result
 
+
 def get_toplevel_window():
     windows = [x for x in gtk.window_list_toplevels()
         if x.window and x.props.visible
-        and x.get_window_type() == gtk.WINDOW_TOPLEVEL]
+        and x.props.type == gtk.WINDOW_TOPLEVEL]
     trans2windows = dict((x.get_transient_for(), x) for x in windows)
     for window in set(windows) - set(trans2windows.iterkeys()):
         return window
     return trans2windows[None]
 
+
 def get_sensible_widget(window):
     from tryton.gui.main import Main
     main = Main.get_main()
@@ -260,6 +347,7 @@ def get_sensible_widget(window):
             return page.widget
     return window
 
+
 def center_window(window, parent, sensible):
     parent_x, parent_y = parent.window.get_origin()
     window_allocation = window.get_allocation()
@@ -270,10 +358,11 @@ def center_window(window, parent, sensible):
         int((sensible_allocation.height - window_allocation.height) / 2))
     window.move(x, y)
 
+
 def selection(title, values, alwaysask=False):
-    if not values or len(values)==0:
+    if not values or len(values) == 0:
         return None
-    elif len(values)==1 and (not alwaysask):
+    elif len(values) == 1 and (not alwaysask):
         key = values.keys()[0]
         return (key, values[key])
 
@@ -331,13 +420,14 @@ def selection(title, values, alwaysask=False):
     dialog.destroy()
     return res
 
+
 def file_selection(title, filename='',
         action=gtk.FILE_CHOOSER_ACTION_OPEN, preview=True, multi=False,
         filters=None):
     parent = get_toplevel_window()
     if action == gtk.FILE_CHOOSER_ACTION_OPEN:
         buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-            gtk.STOCK_OPEN,gtk.RESPONSE_OK)
+            gtk.STOCK_OPEN, gtk.RESPONSE_OK)
     else:
         buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
             gtk.STOCK_SAVE, gtk.RESPONSE_OK)
@@ -383,7 +473,7 @@ def file_selection(title, filename='',
             filepath = filepath.decode('utf-8')
             try:
                 CONFIG['client.default_path'] = \
-                        os.path.dirname(filepath)
+                    os.path.dirname(filepath)
                 CONFIG.save()
             except IOError:
                 pass
@@ -396,55 +486,16 @@ def file_selection(title, filename='',
             filenames = [x.decode('utf-8') for x in filenames]
             try:
                 CONFIG['client.default_path'] = \
-                        os.path.dirname(filenames[0])
+                    os.path.dirname(filenames[0])
             except IOError:
                 pass
         parent.present()
         win.destroy()
         return filenames
 
+
 def file_open(filename, type, print_p=False):
-    if os.name == 'nt':
-        operation = 'open'
-        if print_p:
-            operation = 'print'
-        try:
-            os.startfile(os.path.normpath(filename), operation)
-        except WindowsError:
-            save_name = file_selection(_('Save As...'),
-                    action=gtk.FILE_CHOOSER_ACTION_SAVE)
-            if save_name:
-                file_p = open(filename, 'rb')
-                save_p = open(save_name, 'wb+')
-                save_p.write(file_p.read())
-                save_p.close()
-                file_p.close()
-        return
-    elif sys.platform == 'darwin':
-        pid = os.fork()
-        if not pid:
-            pid = os.fork()
-            if not pid:
-                try:
-                    os.execv('/usr/bin/open', ['/usr/bin/open', filename])
-                except OSError:
-                    sys.exit(0)
-            time.sleep(0.1)
-            sys.exit(0)
-        os.waitpid(pid, 0)
-        return
-    cmd = ''
-    if isinstance(CONFIG['client.actions'], basestring):
-        CONFIG['client.actions'] = safe_eval(CONFIG['client.actions'])
-    if type in CONFIG['client.actions']:
-        if print_p:
-            cmd = CONFIG['client.actions'][type][1]
-        else:
-            cmd = CONFIG['client.actions'][type][0]
-    if not cmd:
-        #TODO add dialog box
-        pass
-    if not cmd:
+    def save():
         save_name = file_selection(_('Save As...'),
                 action=gtk.FILE_CHOOSER_ACTION_SAVE)
         if save_name:
@@ -453,25 +504,26 @@ def file_open(filename, type, print_p=False):
             save_p.write(file_p.read())
             save_p.close()
             file_p.close()
-        return
-    cmd = cmd % filename
-    args = shlex.split(str(cmd))
-    prog = find_in_path(args[0])
-    args[0] = os.path.basename(args[0])
-    if print_p:
-        os.spawnv(os.P_WAIT, prog, args)
-        return
-    pid = os.fork()
-    if not pid:
-        pid = os.fork()
-        if not pid:
-            try:
-                os.execv(prog, args)
-            except OSError:
-                sys.exit(0)
-        time.sleep(0.1)
-        sys.exit(0)
-    os.waitpid(pid, 0)
+
+    if os.name == 'nt':
+        operation = 'open'
+        if print_p:
+            operation = 'print'
+        try:
+            os.startfile(os.path.normpath(filename), operation)
+        except WindowsError:
+            save()
+    elif sys.platform == 'darwin':
+        try:
+            subprocess.Popen(['/usr/bin/open', filename])
+        except OSError:
+            save()
+    else:
+        try:
+            subprocess.Popen(['xdg-open', filename])
+        except OSError:
+            save()
+
 
 def mailto(to=None, cc=None, subject=None, body=None, attachment=None):
     if CONFIG['client.email']:
@@ -483,23 +535,25 @@ def mailto(to=None, cc=None, subject=None, body=None, attachment=None):
                 attachment=attachment or '',
                 )
         args = shlex.split(str(cmd))
-        prog = find_in_path(args[0])
-        args[0] = os.path.basename(args[0])
-        if os.name == 'nt':
-            os.spawnv(os.P_NOWAIT, prog, args)
-            return
-        pid = os.fork()
-        if not pid:
-            pid = os.fork()
-            if not pid:
-                try:
-                    os.execv(prog, args)
-                except OSError:
-                    sys.exit(0)
-            time.sleep(0.1)
-            sys.exit(0)
-        os.waitpid(pid, 0)
+        subprocess.Popen(args)
         return
+    if os.name != 'nt' and sys.platform != 'darwin':
+        args = ['xdg-email', '--utf8']
+        if cc:
+            args.extend(['--cc', cc])
+        if subject:
+            args.extend(['--subject', subject])
+        if body:
+            args.extend(['--body', body])
+        if attachment:
+            args.extend(['--attach', attachment])
+        if to:
+            args.append(to)
+        try:
+            subprocess.Popen(args)
+            return
+        except OSError:
+            pass
     #http://www.faqs.org/rfcs/rfc2368.html
     url = "mailto:"
     if to:
@@ -555,7 +609,7 @@ class MessageDialog(UniqueDialog):
 
     def build_dialog(self, parent, message, msg_type):
         dialog = gtk.MessageDialog(parent,
-            gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT, msg_type,
+            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, msg_type,
             gtk.BUTTONS_OK, message)
         return dialog
 
@@ -574,7 +628,8 @@ class WarningDialog(UniqueDialog):
             dialog.set_markup('<b>%s</b>' % (to_xml(title)))
             dialog.format_secondary_markup(to_xml(message))
         else:
-            dialog.set_markup('<b>%s</b>\n%s' % (to_xml(title), to_xml(message)))
+            dialog.set_markup(
+                '<b>%s</b>\n%s' % (to_xml(title), to_xml(message)))
         return dialog
 
 warning = WarningDialog()
@@ -591,17 +646,20 @@ class UserWarningDialog(WarningDialog):
 
     def build_dialog(self, parent, message, title):
         dialog = super(UserWarningDialog, self).build_dialog(parent, message,
-            title, gtk.BUTTONS_OK_CANCEL)
+            title, gtk.BUTTONS_YES_NO)
         check = gtk.CheckButton(_('Always ignore this warning.'))
         check.connect_after('toggled', self._set_always)
-        alignment = gtk.Alignment(1, 0.5)
+        alignment = gtk.Alignment(0, 0.5)
         alignment.add(check)
-        dialog.vbox.pack_end(alignment, True, False)
+        dialog.vbox.pack_start(alignment, True, False)
+        label = gtk.Label(_('Do you want to proceed?'))
+        label.set_alignment(1, 0.5)
+        dialog.vbox.pack_start(label, True, True)
         return dialog
 
     def __call__(self, message, title):
         response = super(UserWarningDialog, self).__call__(message, title)
-        if response == gtk.RESPONSE_OK:
+        if response == gtk.RESPONSE_YES:
             if self.always:
                 return 'always'
             return 'ok'
@@ -674,7 +732,7 @@ class AskDialog(UniqueDialog):
 
     def build_dialog(self, parent, question, visibility):
         win = gtk.Dialog('Tryton', parent,
-                gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
+                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                 (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                     gtk.STOCK_OK, gtk.RESPONSE_OK))
         win.set_has_separator(True)
@@ -719,9 +777,9 @@ ask = AskDialog()
 class ConcurrencyDialog(UniqueDialog):
 
     def build_dialog(self, parent, resource, obj_id, context):
-        dialog = gtk.Dialog(_('Concurrency Exception'), parent, gtk.DIALOG_MODAL
-                | gtk.DIALOG_DESTROY_WITH_PARENT | gtk.WIN_POS_CENTER_ON_PARENT
-                | gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
+        dialog = gtk.Dialog(_('Concurrency Exception'), parent,
+            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT |
+            gtk.WIN_POS_CENTER_ON_PARENT | gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
         dialog.set_has_separator(True)
         dialog.set_default_response(gtk.RESPONSE_CANCEL)
         hbox = gtk.HBox()
@@ -844,16 +902,17 @@ class ErrorDialog(UniqueDialog):
     def __call__(self, title, details):
         if title == details:
             title = ''
-        log = logging.getLogger('common.message')
-        log.error(details)
+        log = logging.getLogger(__name__)
+        log.error(details + '\n' + title)
 
         response = super(ErrorDialog, self).__call__(title, details)
         if response == gtk.RESPONSE_OK:
-            send_bugtracker(details)
+            send_bugtracker(title, details)
 
 error = ErrorDialog()
 
-def send_bugtracker(msg):
+
+def send_bugtracker(title, msg):
     from tryton import rpc
     parent = get_toplevel_window()
     win = gtk.Dialog(_('Bug Tracker'), parent,
@@ -912,21 +971,15 @@ def send_bugtracker(msg):
             protocol = 'http'
             if ssl or hasattr(socket, 'ssl'):
                 protocol = 'https'
-            server = xmlrpclib.Server(('%s://%s:%s@' + CONFIG['roundup.xmlrpc'])
-                    % (protocol, user, password), allow_none=True)
+            server = xmlrpclib.Server(
+                ('%s://%s:%s@' + CONFIG['roundup.xmlrpc'])
+                % (protocol, user, password), allow_none=True)
             if hashlib:
-                msg_md5 = hashlib.md5(msg).hexdigest()
+                msg_md5 = hashlib.md5(msg + '\n' + title).hexdigest()
             else:
-                msg_md5 = md5.new(msg).hexdigest()
-            # use the last line of the message as title
-            title = '[no title]'
-            for line in msg.splitlines():
-                #don't use empty line nor ^ from sql error
-                if line and '^' != line.strip():
-                    if len(line) > 128:
-                        title = line[:128] + '...'
-                    else:
-                        title = line
+                msg_md5 = md5.new(msg + '\n' + title).hexdigest()
+            if not title:
+                title = '[no title]'
             issue_id = None
             msg_ids = server.filter('msg', None, {'summary': str(msg_md5)})
             if msg_ids:
@@ -936,9 +989,10 @@ def send_bugtracker(msg):
             if issue_id:
                 # issue to same message already exists, add user to nosy-list
                 server.set('issue' + str(issue_id), *['nosy=+' + user])
-                message(_('The same bug was already reported by another user.\n' \
-                        'To keep you informed your username is added to the nosy-list of this issue') + \
-                        '%s' % issue_id)
+                message(
+                    _('The same bug was already reported by another user.\n'
+                        'To keep you informed your username is added '
+                        'to the nosy-list of this issue') + '%s' % issue_id)
             else:
                 # create a new issue for this error-message
                 # first create message
@@ -947,27 +1001,33 @@ def send_bugtracker(msg):
                 # second create issue with this message
                 issue_id = server.create('issue', *['messages=' + str(msg_id),
                     'nosy=' + user, 'title=' + title, 'priority=bug'])
-                message(_('Created new bug with ID ') + \
-                        'issue%s' % issue_id)
-            webbrowser.open(CONFIG['roundup.url'] + 'issue%s' % issue_id, new=2)
+                message(_('Created new bug with ID ')
+                    + 'issue%s' % issue_id)
+            webbrowser.open(CONFIG['roundup.url'] + 'issue%s' % issue_id,
+                new=2)
         except (socket.error, xmlrpclib.Fault), exception:
             if (isinstance(exception, xmlrpclib.Fault)
                     and 'roundup.cgi.exceptions.Unauthorised' in
                     exception.faultString):
                 message(_('Connection error!\nBad username or password!'))
-                return send_bugtracker(msg)
+                return send_bugtracker(title, msg)
             tb_s = reduce(lambda x, y: x + y,
                     traceback.format_exception(sys.exc_type,
                         sys.exc_value, sys.exc_traceback))
             message(_('Exception:') + '\n' + tb_s, msg_type=gtk.MESSAGE_ERROR)
 
+
 def to_xml(string):
-    return string.replace('&','&amp;').replace('<','&lt;').replace('>','&gt;')
+    return string.replace('&', '&amp;'
+        ).replace('<', '&lt;').replace('>', '&gt;')
 
 PLOCK = Lock()
 
+
 def process_exception(exception, *args, **kwargs):
 
+    rpc_execute = kwargs.get('rpc_execute', rpc.execute)
+
     if isinstance(exception, TrytonError):
         if exception.faultCode == 'BadFingerprint':
             warning(
@@ -979,7 +1039,7 @@ def process_exception(exception, *args, **kwargs):
             sys.exit()
         elif exception.faultCode == 'NotLogged':
             if rpc.CONNECTION is None:
-                message(_('Connection error!\n' \
+                message(_('Connection error!\n'
                         'Unable to connect to the server!'))
                 return False
     elif isinstance(exception, TrytonServerError):
@@ -987,20 +1047,21 @@ def process_exception(exception, *args, **kwargs):
             name, msg, description = exception.args
             res = userwarning(description, msg)
             if res in ('always', 'ok'):
-                args2 = ('model', 'res.user.warning', 'create', {
-                        'user': rpc._USER,
-                        'name': name,
-                        'always': (res == 'always'),
-                        }, rpc.CONTEXT)
+                args2 = ('model', 'res.user.warning', 'create', [{
+                            'user': rpc._USER,
+                            'name': name,
+                            'always': (res == 'always'),
+                            }], rpc.CONTEXT)
                 try:
-                    rpc.execute(*args2)
+                    rpc_execute(*args2)
                 except TrytonServerError, exception:
                     process_exception(exception, *args2)
                 if args:
                     try:
-                        return rpc.execute(*args)
+                        return rpc_execute(*args)
                     except TrytonServerError, exception:
-                        return process_exception(exception, *args)
+                        return process_exception(exception, *args,
+                            rpc_execute=rpc_execute)
                 return True
             return False
         elif exception.faultCode == 'UserError':
@@ -1013,15 +1074,24 @@ def process_exception(exception, *args, **kwargs):
                     if '_timestamp' in args[5]:
                         del args[5]['_timestamp']
                     try:
-                        return rpc.execute(*args)
+                        return rpc_execute(*args)
                     except TrytonServerError, exception:
-                        return process_exception(exception, *args)
+                        return process_exception(exception, *args,
+                            rpc_execute=rpc_execute)
                 return False
             else:
                 message(_('Concurrency Exception'), msg_type=gtk.MESSAGE_ERROR)
                 return False
         elif exception.faultCode == 'NotLogged':
             from tryton.gui.main import Main
+            if kwargs.get('session', rpc._SESSION) != rpc._SESSION:
+                if args:
+                    try:
+                        return rpc_execute(*args)
+                    except TrytonServerError, exception:
+                        return process_exception(exception, *args,
+                            rpc_execute=rpc_execute)
+                return
             if not PLOCK.acquire(False):
                 return False
             hostname = rpc._HOST
@@ -1031,21 +1101,22 @@ def process_exception(exception, *args, **kwargs):
                     password = ask(_('Password:'), visibility=False)
                     if password is None:
                         Main.get_main().sig_logout()
-                        raise TrytonError('NotLogged')
+                        return False
                     res = rpc.login(rpc._USERNAME, password, hostname, port,
                             rpc._DATABASE)
                     Main.get_main().refresh_ssl()
                     if res == -1:
-                        message(_('Connection error!\n' \
+                        message(_('Connection error!\n'
                                 'Unable to connect to the server!'))
                         return False
                     if res < 0:
                         continue
                     if args:
                         try:
-                            return rpc.execute(*args)
+                            return rpc_execute(*args)
                         except TrytonServerError, exception:
-                            return process_exception(exception, *args)
+                            return process_exception(exception, *args,
+                                rpc_execute=rpc_execute)
                     return True
             finally:
                 PLOCK.release()
@@ -1064,6 +1135,7 @@ def process_exception(exception, *args, **kwargs):
     error(error_title, error_detail)
     return False
 
+
 def node_attributes(node):
     result = {}
     attrs = node.attributes
@@ -1073,6 +1145,7 @@ def node_attributes(node):
         result[str(attrs.item(i).localName)] = str(attrs.item(i).nodeValue)
     return result
 
+
 def hex2rgb(hexstring, digits=2):
     """
     Converts a hexstring color to a rgb tuple.
@@ -1083,11 +1156,12 @@ def hex2rgb(hexstring, digits=2):
     if isinstance(hexstring, (tuple, list)):
         return hexstring
     top = float(int(digits * 'f', 16))
-    r = int(hexstring[1:digits+1], 16)
-    g = int(hexstring[digits+1:digits*2+1], 16)
-    b = int(hexstring[digits*2+1:digits*3+1], 16)
+    r = int(hexstring[1:digits + 1], 16)
+    g = int(hexstring[digits + 1:digits * 2 + 1], 16)
+    b = int(hexstring[digits * 2 + 1:digits * 3 + 1], 16)
     return r / top, g / top, b / top
 
+
 def clamp(minValue, maxValue, value):
     """Make sure value is between minValue and maxValue"""
     if value < minValue:
@@ -1096,12 +1170,14 @@ def clamp(minValue, maxValue, value):
                 return maxValue
     return value
 
+
 def lighten(r, g, b, amount):
     """Return a lighter version of the color (r, g, b)"""
     return (clamp(0.0, 1.0, r + amount),
             clamp(0.0, 1.0, g + amount),
             clamp(0.0, 1.0, b + amount))
 
+
 def generateColorscheme(masterColor, keys, light=0.06):
     """
     Generates a dictionary where the keys match the keys argument and
@@ -1136,7 +1212,7 @@ class DBProgress(object):
 
     def update(self, combo, progressbar, callback, dbname=''):
         self.db_info = None
-        thread = threading.Thread(target=self.start).start()
+        threading.Thread(target=self.start).start()
         gobject.timeout_add(100, self.end, combo, progressbar, callback,
             dbname)
 
@@ -1171,6 +1247,12 @@ class DBProgress(object):
         return False
 
 
+class RPCException(Exception):
+
+    def __init__(self, exception):
+        self.exception = exception
+
+
 class RPCProgress(object):
 
     def __init__(self, method, args):
@@ -1193,72 +1275,49 @@ class RPCProgress(object):
             self.error = True
         return True
 
-    def run(self):
+    def run(self, process_exception_p=True, main_iteration_p=True):
+        session = rpc._SESSION
         thread.start_new_thread(self.start, ())
 
-        watch = gtk.gdk.Cursor(gtk.gdk.WATCH)
-        self.parent.window.set_cursor(watch)
-        parent_sensitive = self.parent.props.sensitive
-        self.parent.props.sensitive = False
+        watch = None
         i = 0
-        win = None
-        progressbar = None
         while (not self.res) and (not self.error):
             i += 1
-            if i > 10:
-                if not win or not progressbar:
-                    win = gtk.Window(type=gtk.WINDOW_TOPLEVEL)
-                    win.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
-                    if hasattr(win, 'set_deletable'):
-                        win.set_deletable(False)
-                    win.set_decorated(False)
-                    vbox = gtk.VBox(False, 0)
-                    hbox = gtk.HBox(False, 13)
-                    hbox.set_border_width(10)
-                    img = gtk.Image()
-                    img.set_from_stock('tryton-dialog-information',
-                            gtk.ICON_SIZE_DIALOG)
-                    hbox.pack_start(img, expand=True, fill=False)
-                    vbox2 = gtk.VBox(False, 0)
-                    label = gtk.Label()
-                    label.set_markup('<b>'+_('Operation in progress')+'</b>')
-                    label.set_alignment(0.0, 0.5)
-                    vbox2.pack_start(label, expand=True, fill=False)
-                    vbox2.pack_start(gtk.HSeparator(), expand=True, fill=True)
-                    vbox2.pack_start(gtk.Label(_("Please wait,\n" \
-                            "this operation may take a while...")),
-                            expand=True, fill=False)
-                    hbox.pack_start(vbox2, expand=True, fill=True)
-                    vbox.pack_start(hbox)
-                    progressbar = gtk.ProgressBar()
-                    progressbar.set_orientation(gtk.PROGRESS_LEFT_TO_RIGHT)
-                    vbox.pack_start(progressbar, expand=True, fill=False)
-                    viewport = gtk.Viewport()
-                    viewport.set_shadow_type(gtk.SHADOW_ETCHED_IN)
-                    viewport.add(vbox)
-                    win.add(viewport)
-                    win.set_transient_for(self.parent)
-                    win.set_modal(True)
-                    win.show_all()
-                    win.window.set_cursor(watch)
-                with gtk.gdk.lock:
-                    progressbar.pulse()
-            if win:
-                with gtk.gdk.lock:
-                    while gtk.events_pending():
-                        gtk.main_iteration()
+            if i > 1:
+                watch = gtk.gdk.Cursor(gtk.gdk.WATCH)
+                self.parent.window.set_cursor(watch)
+                if main_iteration_p:
+                    with gtk.gdk.lock:
+                        while gtk.events_pending():
+                            gtk.main_iteration()
             time.sleep(0.1)
-        self.parent.props.sensitive = parent_sensitive
-        self.parent.window.set_cursor(None)
-        if win:
-            win.destroy()
-            while gtk.events_pending():
-                gtk.main_iteration()
+        if self.parent.window:
+            self.parent.window.set_cursor(None)
         if self.exception:
-            raise self.exception
+            if process_exception_p:
+                def rpc_execute(*args):
+                    return RPCProgress('execute',
+                        args).run(process_exception_p, main_iteration_p)
+                result = process_exception(self.exception, *self.args,
+                    rpc_execute=rpc_execute, session=session)
+                if result is False:
+                    raise RPCException(self.exception)
+                return result
+            else:
+                raise self.exception
         return self.res
 
 
+def RPCExecute(*args, **kwargs):
+    rpc_context = rpc.CONTEXT.copy()
+    if kwargs.get('context'):
+        rpc_context.update(kwargs['context'])
+    args = args + (rpc_context,)
+    process_exception = kwargs.get('process_exception', True)
+    main_iteration = kwargs.get('main_iteration', True)
+    return RPCProgress('execute', args).run(process_exception, main_iteration)
+
+
 class Tooltips(object):
     _tooltips = None
 
@@ -1287,21 +1346,17 @@ COLOR_SCHEMES = {
 }
 
 COLORS = {
-    'invalid':'#ff6969',
-    'required':'#d2d2ff',
+    'invalid': '#ff6969',
+    'required': '#d2d2ff',
 }
 
-DT_FORMAT = '%Y-%m-%d'
-HM_FORMAT = '%H:%M:%S'
-DHM_FORMAT = DT_FORMAT + ' ' + HM_FORMAT
-
 FLOAT_TIME_CONV = {
     'Y': 8760,
     'M': 672,
     'w': 168,
     'd': 24,
     'h': 1,
-    'm': 1.0/60,
+    'm': 1.0 / 60,
 }
 
 FLOAT_TIME_SEPS = {
@@ -1313,10 +1368,13 @@ FLOAT_TIME_SEPS = {
     'm': _('m'),
 }
 
-def text_to_float_time(text, conv=None):
+
+def text_to_float_time(text, conv=None, digit=2):
+    if not text:
+        return None
     try:
         try:
-            return locale.atof(text)
+            return round(locale.atof(text), digit)
         except ValueError:
             pass
         if conv:
@@ -1351,11 +1409,14 @@ def text_to_float_time(text, conv=None):
                     break
         if text.startswith('-'):
             value *= -1
-        return value
+        return round(value, digit)
     except ValueError:
         return 0.0
 
+
 def float_time_to_text(val, conv=None):
+    if val is None:
+        return ''
     if conv:
         tmp_conv = FLOAT_TIME_CONV.copy()
         tmp_conv.update(conv)
@@ -1377,7 +1438,7 @@ def float_time_to_text(val, conv=None):
     val = val - days * conv['d']
     hours = int(val)
     val = val - hours
-    mins = int((val% 1 + 0.01) / conv['m'])
+    mins = int((val % 1 + 0.01) / conv['m'])
     if years:
         value += ' ' + locale.format('%d', years, True) + FLOAT_TIME_SEPS['Y']
     if months:
@@ -1386,11 +1447,12 @@ def float_time_to_text(val, conv=None):
         value += ' ' + locale.format('%d', weeks, True) + FLOAT_TIME_SEPS['w']
     if days:
         value += ' ' + locale.format('%d', days, True) + FLOAT_TIME_SEPS['d']
-    if hours or mins:
+    if hours or mins or not value:
         value += ' %02d:%02d' % (hours, mins)
     value = value.strip()
     return value
 
+
 def filter_domain(domain):
     '''
     Return the biggest subset of domain with only AND operator
@@ -1408,59 +1470,6 @@ def filter_domain(domain):
             res.extend(filter_domain(arg))
     return res
 
-_ALLOWED_CODES = set(dis.opmap[x] for x in [
-    'POP_TOP','ROT_TWO','ROT_THREE','ROT_FOUR','DUP_TOP',
-    'BUILD_LIST','BUILD_MAP','BUILD_TUPLE',
-    'LOAD_CONST','RETURN_VALUE','STORE_SUBSCR',
-    'UNARY_POSITIVE','UNARY_NEGATIVE','UNARY_NOT',
-    'UNARY_INVERT','BINARY_POWER','BINARY_MULTIPLY',
-    'BINARY_DIVIDE','BINARY_FLOOR_DIVIDE','BINARY_TRUE_DIVIDE',
-    'BINARY_MODULO','BINARY_ADD','BINARY_SUBTRACT',
-    'BINARY_LSHIFT','BINARY_RSHIFT','BINARY_AND','BINARY_XOR', 'BINARY_OR',
-    'STORE_MAP', 'LOAD_NAME', 'CALL_FUNCTION', 'COMPARE_OP', 'LOAD_ATTR',
-    'STORE_NAME', 'GET_ITER', 'FOR_ITER', 'LIST_APPEND', 'JUMP_ABSOLUTE',
-    'DELETE_NAME', 'JUMP_IF_TRUE', 'JUMP_IF_FALSE', 'JUMP_IF_FALSE_OR_POP',
-    'JUMP_IF_TRUE_OR_POP', 'POP_JUMP_IF_FALSE', 'POP_JUMP_IF_TRUE',
-    'BINARY_SUBSCR',
-    ] if x in dis.opmap)
-
-_SAFE_EVAL_CACHE = {}
-
-def safe_eval(source, data=None):
-    if '__subclasses__' in source:
-        raise ValueError('__subclasses__ not allowed')
-    if hashlib:
-        key = hashlib.md5(source).digest()
-    else:
-        key = md5.new(source).digest()
-    c = _SAFE_EVAL_CACHE.get(key)
-    if not c:
-        c = compile(source, '', 'eval')
-        codes = []
-        s = c.co_code
-        i = 0
-        while i < len(s):
-            code = ord(s[i])
-            codes.append(code)
-            if code >= dis.HAVE_ARGUMENT:
-                i += 3
-            else:
-                i += 1
-        for code in codes:
-            if code not in _ALLOWED_CODES:
-                raise ValueError('opcode %s not allowed' % dis.opname[code])
-        if len(_SAFE_EVAL_CACHE) > 1024:
-            _SAFE_EVAL_CACHE.clear()
-        _SAFE_EVAL_CACHE[key] = c
-    return eval(c, {'__builtins__': {
-        'True': True,
-        'False': False,
-        'str': str,
-        'globals': locals,
-        'locals': locals,
-        'bool': bool,
-        'dict': dict,
-        }}, data)
 
 def timezoned_date(date, reverse=False):
     if pytz and rpc.CONTEXT.get('timezone') and rpc.TIMEZONE:
@@ -1476,3 +1485,10 @@ def timezoned_date(date, reverse=False):
 
 def untimezoned_date(date):
     return timezoned_date(date, reverse=True).replace(tzinfo=None)
+
+
+def humanize(size):
+    for x in ('bytes', 'KB', 'MB', 'GB', 'TB', 'PB'):
+        if size < 1000:
+            return '%3.1f%s' % (size, x)
+        size /= 1000.0
