@@ -4,12 +4,9 @@ import tryton.rpc as rpc
 from tryton.signal_event import SignalEvent
 import tryton.common as common
 from tryton.pyson import PYSONDecoder
-import field
-import datetime
-import logging
-import time
+import field as fields
 from functools import reduce
-from tryton.exceptions import TrytonServerError
+from tryton.common import RPCExecute, RPCException
 
 
 class Record(SignalEvent):
@@ -33,71 +30,93 @@ class Record(SignalEvent):
         self.modified_fields = {}
         self._timestamp = None
         self.attachment_count = -1
-        self.next = {} # Used in Group list
+        self.next = {}  # Used in Group list
         self.value = {}
         self.autocompletion = {}
+        self.exception = False
+        self.destroyed = False
+        self.pool.add(self)
 
-    def __getitem__(self, name, raise_exception=False):
+    def __getitem__(self, name):
         if name not in self._loaded and self.id >= 0:
-            ids =  [self.id]
+            id2record = {
+                self.id: self,
+                }
             if name == '*':
                 loading = reduce(
                         lambda x, y: 'eager' if x == y == 'eager' else 'lazy',
                         (field.attrs.get('loading', 'eager')
                             for field in self.group.fields.itervalues()),
                         'eager')
+                # Set a valid name for next loaded check
+                for fname, field in self.group.fields.iteritems():
+                    if field.attrs.get('loading', 'eager') == loading:
+                        name = fname
+                        break
             else:
                 loading = self.group.fields[name].attrs.get('loading', 'eager')
             if self in self.group and loading == 'eager':
                 idx = self.group.index(self)
                 length = len(self.group)
                 n = 1
-                while len(ids) < 80 and (idx - n >= 0 or \
-                        idx + n < length) and n < 100:
+                while len(id2record) < 80 and (idx - n >= 0
+                        or idx + n < length) and n < 100:
                     if idx - n >= 0:
                         record = self.group[idx - n]
                         if name not in record._loaded and record.id >= 0:
-                            ids.append(record.id)
+                            id2record[record.id] = record
                     if idx + n < length:
                         record = self.group[idx + n]
                         if name not in record._loaded and record.id >= 0:
-                            ids.append(record.id)
+                            id2record[record.id] = record
                     n += 1
+            record_context = self.context_get()
+            if loading == 'eager' and len(id2record) < 80:
+                for record in self.pool:
+                    if (name not in record._loaded
+                            and record.id >= 0
+                            and record.id not in id2record
+                            and record.context_get() == record_context):
+                        id2record[record.id] = record
+                        if len(id2record) == 80:
+                            break
             if loading == 'eager':
-                fields = [fname for fname, field in self.group.fields.iteritems()
-                        if field.attrs.get('loading', 'eager') == 'eager']
+                fnames = [fname
+                    for fname, field in self.group.fields.iteritems()
+                    if field.attrs.get('loading', 'eager') == 'eager']
             else:
-                fields = self.group.fields.keys()
-            fields = [fname for fname in fields if fname not in self._loaded]
-            fields.extend(('%s.rec_name' % fname for fname in fields[:]
+                fnames = self.group.fields.keys()
+            fnames = [fname for fname in fnames if fname not in self._loaded]
+            fnames.extend(('%s.rec_name' % fname for fname in fnames[:]
                     if self.group.fields[fname].attrs['type']
                     in ('many2one', 'one2one', 'reference')))
-            fields.append('_timestamp')
-            ctx = rpc.CONTEXT.copy()
-            ctx.update(self.context_get())
+            if 'rec_name' not in fnames:
+                fnames.append('rec_name')
+            fnames.append('_timestamp')
+            ctx = record_context.copy()
             ctx.update(dict(('%s.%s' % (self.model_name, fname), 'size')
                     for fname, field in self.group.fields.iteritems()
-                    if field.attrs['type'] == 'binary'))
-            args = ('model', self.model_name, 'read', ids, fields, ctx)
+                    if field.attrs['type'] == 'binary' and fname in fnames))
+            exception = None
             try:
-                values = rpc.execute(*args)
-            except TrytonServerError, exception:
-                if raise_exception:
-                    raise
-                values = common.process_exception(exception, *args)
-                if not values:
-                    values = [{'id': x} for x in ids]
+                values = RPCExecute('model', self.model_name, 'read',
+                    id2record.keys(), fnames, context=ctx,
+                    main_iteration=False)
+            except RPCException, exception:
+                values = [{'id': x} for x in id2record]
+                default_values = dict((f, None) for f in fnames)
+                for value in values:
+                    value.update(default_values)
+                self.exception = True
             id2value = dict((value['id'], value) for value in values)
-            if ids != [self.id]:
-                for id in ids:
-                    record = self.group.get(id)
-                    value = id2value.get(id)
-                    if record and value:
-                        record.set(value, signal=False)
-            else:
-                value = id2value.get(self.id)
-                if value:
-                    self.set(value, signal=False)
+            for id, record in id2record.iteritems():
+                if not record.exception:
+                    record.exception = bool(exception)
+                value = id2value.get(id)
+                if record and not record.destroyed and value:
+                    for key in record.modified_fields:
+                        value.pop(key, None)
+                    record.set(value, signal=False)
         return self.group.fields.get(name, False)
 
     def __repr__(self):
@@ -107,6 +126,14 @@ class Record(SignalEvent):
     def modified(self):
         return bool(self.modified_fields)
 
+    @property
+    def pool(self):
+        group = self.group
+        while (group.parent is not None
+                and group.parent.model_name == self.model_name):
+            group = group.parent
+        return group.pool
+
     @property
     def parent(self):
         return self.group.parent
@@ -115,18 +142,26 @@ class Record(SignalEvent):
     def parent_name(self):
         return self.group.parent_name
 
+    @property
+    def depth(self):
+        parent = self.parent
+        i = 0
+        while parent:
+            i += 1
+            parent = parent.parent
+        return i
+
     def set_modified(self, value):
         if value:
             self.signal('record-modified')
 
-    def children_group(self, field_name, check_load=True):
+    def children_group(self, field_name):
         if not field_name:
             return []
-        if check_load:
-            self._check_load([field_name])
+        self._check_load([field_name])
         group = self.value.get(field_name)
-        if group is False or group is None:
-            return []
+        if group is None:
+            return None
 
         if id(group.fields) != id(self.group.fields):
             self.group.fields.update(group.fields)
@@ -164,7 +199,7 @@ class Record(SignalEvent):
     deleted = property(get_deleted)
 
     def get_readonly(self):
-        return self.deleted or self.removed
+        return self.deleted or self.removed or self.exception
 
     readonly = property(get_readonly)
 
@@ -184,38 +219,36 @@ class Record(SignalEvent):
 
     def get_loaded(self, fields=None):
         if fields:
-            return set(fields) <= self._loaded
+            return set(fields) <= (self._loaded | set(self.modified_fields))
         return set(self.group.fields.keys()) == self._loaded
 
     loaded = property(get_loaded)
 
-    def get(self, get_readonly=True, includeid=False, check_load=True,
-            get_modifiedonly=False):
-        if check_load:
-            self._check_load()
-        value = []
+    def get(self):
+        value = {}
         for name, field in self.group.fields.iteritems():
             if field.attrs.get('readonly'):
                 continue
-            if (field.get_state_attrs(self).get('readonly', False)
-                    and not get_readonly):
+            if field.name not in self.modified_fields and self.id >= 0:
                 continue
-            if (field.name not in self.modified_fields
-                    and get_modifiedonly):
-                continue
-            value.append((name, field.get(self, check_load=check_load,
-                readonly=get_readonly, modified=get_modifiedonly)))
-        value = dict(value)
-        if includeid:
-            value['id'] = self.id
+            value[name] = field.get(self)
         return value
 
-    def get_eval(self, check_load=True):
-        if check_load:
-            self._check_load()
+    def get_eval(self):
         value = {}
         for name, field in self.group.fields.iteritems():
-            value[name] = field.get_eval(self, check_load=check_load)
+            if name not in self._loaded and self.id >= 0:
+                continue
+            value[name] = field.get_eval(self)
+        value['id'] = self.id
+        return value
+
+    def get_on_change_value(self):
+        value = {}
+        for name, field in self.group.fields.iteritems():
+            if name not in self._loaded and self.id >= 0:
+                continue
+            value[name] = field.get_on_change_value(self)
         value['id'] = self.id
         return value
 
@@ -229,38 +262,40 @@ class Record(SignalEvent):
             result.update(field.get_timestamp(self))
         return result
 
+    def pre_validate(self):
+        values = self.get()
+        try:
+            RPCExecute('model', self.model_name, 'pre_validate', values,
+                main_iteration=False, context=self.context_get())
+        except RPCException:
+            return False
+        return True
+
     def save(self, force_reload=True):
         if self.id < 0 or self.modified:
             if self.id < 0:
-                value = self.get(get_readonly=True)
-                args = ('model', self.model_name, 'create', value,
-                    self.context_get())
+                value = self.get()
                 try:
-                    res = rpc.execute(*args)
-                except TrytonServerError, exception:
-                    res = common.process_exception(exception, *args)
-                    if not res:
-                        return False
+                    res, = RPCExecute('model', self.model_name, 'create',
+                        [value], main_iteration=False,
+                        context=self.context_get())
+                except RPCException:
+                    return False
                 old_id = self.id
                 self.id = res
                 self.group.id_changed(old_id)
             elif self.modified:
-                self._check_load()
-                value = self.get(get_readonly=True, get_modifiedonly=True,
-                        check_load=False)
+                value = self.get()
                 if value:
                     context = self.context_get()
                     context = context.copy()
                     context['_timestamp'] = self.get_timestamp()
-                    args = ('model', self.model_name, 'write', [self.id],
-                            value, context)
                     try:
-                        if not rpc.execute(*args):
-                            return False
-                    except TrytonServerError, exception:
-                        res = common.process_exception(exception, *args)
-                        if not res:
-                            return False
+                        RPCExecute('model', self.model_name, 'write',
+                            [self.id], value, main_iteration=False,
+                            context=context)
+                    except RPCException:
+                        return False
             self._loaded.clear()
             self.modified_fields = {}
             if force_reload:
@@ -273,7 +308,7 @@ class Record(SignalEvent):
         return self.id
 
     @staticmethod
-    def delete(records, context=None):
+    def delete(records):
         if not records:
             return
         record = records[0]
@@ -282,8 +317,6 @@ class Record(SignalEvent):
         assert all(r.group.root_group == root_group for r in records)
         records = [r for r in records if r.id >= 0]
         ctx = {}
-        ctx.update(rpc.CONTEXT)
-        ctx.update(context or {})
         ctx['_timestamp'] = {}
         for rec in records:
             ctx['_timestamp'].update(rec.get_timestamp())
@@ -291,31 +324,32 @@ class Record(SignalEvent):
         reload_ids = set(root_group.on_write_ids(list(record_ids)))
         reload_ids -= record_ids
         reload_ids = list(reload_ids)
-        args = ('model', record.model_name, 'delete', list(record_ids), ctx)
         try:
-            rpc.execute(*args)
-        except TrytonServerError, exception:
-            if not common.process_exception(exception, *args):
-                return False
+            RPCExecute('model', record.model_name, 'delete', list(record_ids),
+                main_iteration=False, context=ctx)
+        except RPCException:
+            return False
         if reload_ids:
             root_group.reload(reload_ids)
         return True
 
-    def default_get(self, domain=None, context=None):
+    def default_get(self):
         if len(self.group.fields):
-            args = ('model', self.model_name, 'default_get',
-                    self.group.fields.keys(), context)
             try:
-                vals = rpc.execute(*args)
-            except TrytonServerError, exception:
-                vals = common.process_exception(exception, *args)
-                if not vals:
-                    return
+                vals = RPCExecute('model', self.model_name, 'default_get',
+                    self.group.fields.keys(), main_iteration=False,
+                    context=self.context_get())
+            except RPCException:
+                return
             if (self.parent
-                    and self.parent_name in self.group.fields
-                    and (self.group.fields[self.parent_name].attrs['relation']
-                        == self.group.parent.model_name)):
-                vals[self.parent_name] = self.parent.id
+                    and self.parent_name in self.group.fields):
+                parent_field = self.group.fields[self.parent_name]
+                if isinstance(parent_field, fields.ReferenceField):
+                    vals[self.parent_name] = (
+                        self.parent.model_name, self.parent.id)
+                elif (self.group.fields[self.parent_name].attrs['relation']
+                        == self.group.parent.model_name):
+                    vals[self.parent_name] = self.parent.id
             self.set_default(vals)
         for fieldname, fieldinfo in self.group.fields.iteritems():
             if not fieldinfo.attrs.get('autocomplete'):
@@ -323,16 +357,12 @@ class Record(SignalEvent):
             self.do_autocomplete(fieldname)
 
     def rec_name(self):
-        ctx = rpc.CONTEXT.copy()
-        ctx.update(self.context_get())
-        args = ('model', self.model_name, 'read', self.id, ['rec_name'], ctx)
         try:
-            res = rpc.execute(*args)
-        except TrytonServerError, exception:
-            res = common.process_exception(exception, *args)
-            if not res:
-                return ''
-        return res['rec_name']
+            return RPCExecute('model', self.model_name, 'read', [self.id],
+                ['rec_name'], main_iteration=False,
+                context=self.context_get())[0]['rec_name']
+        except RPCException:
+            return ''
 
     def validate(self, fields=None, softvalidation=False):
         if isinstance(fields, list) and fields:
@@ -363,68 +393,76 @@ class Record(SignalEvent):
     def context_get(self):
         return self.group.context
 
-    def get_default(self):
-        self._check_load()
-        value = dict([(name, field.get_default(self))
-                      for name, field in self.group.fields.iteritems()])
-        return value
-
-    def set_default(self, val, signal=True, modified=False):
+    def set_default(self, val, signal=True):
         for fieldname, value in val.items():
             if fieldname not in self.group.fields:
                 continue
-            if isinstance(self.group.fields[fieldname], field.M2OField):
-                if fieldname + '.rec_name' in val:
-                    value = (value, val[fieldname + '.rec_name'])
-            elif isinstance(self.group.fields[fieldname], field.ReferenceField):
-                if value:
-                    ref_model, ref_id = value.split(',', 1)
-                    if fieldname + '.rec_name' in val:
-                        value = ref_model, (ref_id,
-                            val[fieldname + '.rec_name'])
-                    else:
-                        value = ref_model, (ref_id, ref_id)
-            self.group.fields[fieldname].set_default(self, value,
-                modified=modified)
+            if isinstance(self.group.fields[fieldname], (fields.M2OField,
+                        fields.ReferenceField)):
+                field_rec_name = fieldname + '.rec_name'
+                if field_rec_name in val:
+                    self.value[field_rec_name] = val[field_rec_name]
+                elif field_rec_name in self.value:
+                    del self.value[field_rec_name]
+            self.group.fields[fieldname].set_default(self, value)
             self._loaded.add(fieldname)
         self.validate(softvalidation=True)
         if signal:
             self.signal('record-changed')
 
-    def set(self, val, modified=False, signal=True):
+    def set(self, val, signal=True):
         later = {}
         for fieldname, value in val.iteritems():
             if fieldname == '_timestamp':
                 self._timestamp = value
                 continue
             if fieldname not in self.group.fields:
+                if fieldname == 'rec_name':
+                    self.value['rec_name'] = value
                 continue
-            if isinstance(self.group.fields[fieldname], field.O2MField):
+            if isinstance(self.group.fields[fieldname], fields.O2MField):
                 later[fieldname] = value
                 continue
-            if isinstance(self.group.fields[fieldname], field.M2OField):
-                if fieldname + '.rec_name' in val:
-                    value = (value, val[fieldname + '.rec_name'])
-            elif isinstance(self.group.fields[fieldname], field.ReferenceField):
-                if value:
-                    ref_model, ref_id = value.split(',', 1)
-                    if fieldname + '.rec_name' in val:
-                        value = ref_model, (ref_id,
-                            val[fieldname + '.rec_name'])
-                    else:
-                        value = ref_model, (ref_id, ref_id)
-            self.group.fields[fieldname].set(self, value, modified=False)
+            if isinstance(self.group.fields[fieldname], (fields.M2OField,
+                        fields.ReferenceField)):
+                field_rec_name = fieldname + '.rec_name'
+                if field_rec_name in val:
+                    self.value[field_rec_name] = val[field_rec_name]
+                elif field_rec_name in self.value:
+                    del self.value[field_rec_name]
+            self.group.fields[fieldname].set(self, value)
             self._loaded.add(fieldname)
         for fieldname, value in later.iteritems():
-            self.group.fields[fieldname].set(self, value, modified=False)
+            self.group.fields[fieldname].set(self, value)
             self._loaded.add(fieldname)
-        if modified:
-            self.modified_fields.update(dict((x, None) for x in val))
-            self.signal('record-modified')
-            self.signal('record-changed')
         if signal:
             self.signal('record-changed')
 
+    def set_on_change(self, values):
+        later = {}
+        for fieldname, value in values.items():
+            if fieldname not in self.group.fields:
+                continue
+            if isinstance(self.group.fields[fieldname], fields.O2MField):
+                later[fieldname] = value
+                continue
+            if isinstance(self.group.fields[fieldname], (fields.M2OField,
+                        fields.ReferenceField)):
+                field_rec_name = fieldname + '.rec_name'
+                if field_rec_name in values:
+                    self.value[field_rec_name] = values[field_rec_name]
+                elif field_rec_name in self.value:
+                    del self.value[field_rec_name]
+            self.group.fields[fieldname].set_on_change(self, value)
+        for fieldname, value in later.items():
+            # on change recursion checking is done only for x2many
+            field_x2many = self.group.fields[fieldname]
+            try:
+                field_x2many.in_on_change = True
+                field_x2many.set_on_change(self, value)
+            finally:
+                field_x2many.in_on_change = False
+
     def reload(self, fields=None):
         if self.id < 0:
             return
@@ -433,29 +471,27 @@ class Record(SignalEvent):
         else:
             for field in fields:
                 self[field]
-        self.validate([])
+        self.validate(fields or [])
 
-    def expr_eval(self, expr, check_load=False):
+    def expr_eval(self, expr):
         if not isinstance(expr, basestring):
             return expr
-        if check_load:
-            self._check_load()
         ctx = rpc.CONTEXT.copy()
-        for name, field in self.group.fields.items():
-            ctx[name] = field.get_eval(self, check_load=check_load)
-
-        ctx['context'] = self.context_get()
+        ctx['context'] = ctx.copy()
+        ctx['context'].update(self.context_get())
+        ctx.update(self.get_eval())
+        ctx['active_model'] = self.model_name
         ctx['active_id'] = self.id
         ctx['_user'] = rpc._USER
         if self.parent and self.parent_name:
             ctx['_parent_' + self.parent_name] = \
-                    common.EvalEnvironment(self.parent, check_load)
+                common.EvalEnvironment(self.parent)
         val = PYSONDecoder(ctx).decode(expr)
         return val
 
     def _get_on_change_args(self, args):
         res = {}
-        values = common.EvalEnvironment(self, True, 'on_change')
+        values = common.EvalEnvironment(self, 'on_change')
         for arg in args:
             scope = values
             for i in arg.split('.'):
@@ -470,45 +506,18 @@ class Record(SignalEvent):
         if isinstance(attr, basestring):
             attr = PYSONDecoder().decode(attr)
         args = self._get_on_change_args(attr)
-        ctx = rpc.CONTEXT.copy()
-        ctx.update(self.context_get())
-        args = ('model', self.model_name, 'on_change_' + fieldname, args, ctx)
         try:
-            res = rpc.execute(*args)
-        except TrytonServerError, exception:
-            res = common.process_exception(exception, *args)
-            if not res:
-                return
-        later = {}
-        for fieldname, value in res.items():
-            if fieldname not in self.group.fields:
-                continue
-            if isinstance(self.group.fields[fieldname], field.O2MField):
-                later[fieldname] = value
-                continue
-            if isinstance(self.group.fields[fieldname], field.M2OField):
-                if fieldname + '.rec_name' in res:
-                    value = (value, res[fieldname + '.rec_name'])
-            elif isinstance(self.group.fields[fieldname],
-                    field.ReferenceField):
-                if value:
-                    ref_model, ref_id = value.split(',', 1)
-                    if fieldname + '.rec_name' in res:
-                        value = ref_model, (ref_id,
-                                res[fieldname + '.rec_name'])
-                    else:
-                        value = ref_model, (ref_id, ref_id)
-            self.group.fields[fieldname].set_on_change(self, value)
-        for fieldname, value in later.items():
-            # on change recursion checking is done only for x2many
-            field_x2many = self.group.fields[fieldname]
-            try:
-                field_x2many.in_on_change = True
-                field_x2many.set_on_change(self, value)
-            finally:
-                field_x2many.in_on_change = False
+            res = RPCExecute('model', self.model_name, 'on_change_' +
+                fieldname, args, main_iteration=False,
+                context=self.context_get())
+        except RPCException:
+            return
+        self.set_on_change(res)
 
     def on_change_with(self, field_name):
+        fieldnames = set()
+        values = {}
+        later = set()
         for fieldname in self.group.fields:
             on_change_with = self.group.fields[fieldname].attrs.get(
                     'on_change_with')
@@ -518,18 +527,35 @@ class Record(SignalEvent):
                 continue
             if field_name == fieldname:
                 continue
-            args = self._get_on_change_args(on_change_with)
-            ctx = rpc.CONTEXT.copy()
-            ctx.update(self.context_get())
-            args = ('model', self.model_name, 'on_change_with_' + fieldname,
-                    args, ctx)
+            if fieldnames & set(on_change_with):
+                later.add(fieldname)
+                continue
+            fieldnames.add(fieldname)
+            values.update(self._get_on_change_args(on_change_with))
+            if isinstance(self.group.fields[fieldname], (fields.M2OField,
+                        fields.ReferenceField)):
+                field_rec_name = fieldname + '.rec_name'
+                if field_rec_name in self.value:
+                    del self.value[field_rec_name]
+        if fieldnames:
             try:
-                res = rpc.execute(*args)
-            except TrytonServerError, exception:
-                res = common.process_exception(exception, *args)
-                if not res:
-                    return
-            self.group.fields[fieldname].set_on_change(self, res)
+                result = RPCExecute('model', self.model_name, 'on_change_with',
+                    values, list(fieldnames), main_iteration=False,
+                    context=self.context_get())
+            except RPCException:
+                return
+            self.set_on_change(result)
+        for fieldname in later:
+            on_change_with = self.group.fields[fieldname].attrs.get(
+                    'on_change_with')
+            values = self._get_on_change_args(on_change_with)
+            try:
+                result = RPCExecute('model', self.model_name,
+                    'on_change_with_' + fieldname, values,
+                    main_iteration=False, context=self.context_get())
+            except RPCException:
+                return
+            self.group.fields[fieldname].set_on_change(self, result)
 
     def autocomplete_with(self, field_name):
         for fieldname, fieldinfo in self.group.fields.iteritems():
@@ -542,50 +568,38 @@ class Record(SignalEvent):
         self.autocompletion[fieldname] = []
         autocomplete = self.group.fields[fieldname].attrs['autocomplete']
         args = self._get_on_change_args(autocomplete)
-        ctx = rpc.CONTEXT.copy()
-        ctx.update(self.context_get())
-        args = ('model', self.model_name, 'autocomplete_' + fieldname, args,
-            ctx)
         try:
-            res = rpc.execute(*args)
-        except TrytonServerError, exception:
-            res = common.process_exception(exception, *args)
-            if not res:
-                # ensure res is a list
-                res = []
+            res = RPCExecute('model', self.model_name, 'autocomplete_' +
+                fieldname, args, main_iteration=False,
+                context=self.context_get())
+        except RPCException:
+            # ensure res is a list
+            res = []
         self.autocompletion[fieldname] = res
 
-    def cond_default(self, field_name, value):
-        ctx = rpc.CONTEXT.copy()
-        ctx.update(self.context_get())
-        args = ('model', 'ir.default', 'get_default', self.model_name,
-                field_name + '=' + str(value), ctx)
-        try:
-            res = rpc.execute(*args)
-        except TrytonServerError, exception:
-            res = common.process_exception(exception, *args)
-            if not res:
-                return
-        self.set_default(res)
-
     def get_attachment_count(self, reload=False):
         if self.id < 0:
             return 0
         if self.attachment_count < 0 or reload:
-            args = ('model', 'ir.attachment', 'search_count', [
-                ('resource', '=', '%s,%s' % (self.model_name, self.id)),
-                ], rpc.CONTEXT)
             try:
-                self.attachment_count = rpc.execute(*args)
-            except TrytonServerError:
+                self.attachment_count = RPCExecute('model', 'ir.attachment',
+                    'search_count', [
+                        ('resource', '=',
+                            '%s,%s' % (self.model_name, self.id)),
+                        ], main_iteration=False)
+            except RPCException:
                 return 0
         return self.attachment_count
 
     def destroy(self):
-        super(Record, self).destroy()
-        self.group = None
+        # Get reference to pool before unref group
+        pool = self.pool
         for v in self.value.itervalues():
             if hasattr(v, 'destroy'):
                 v.destroy()
+        super(Record, self).destroy()
+        self.group = None
         self.value = None
         self.next = None
+        self.destroyed = True
+        pool.remove(self)
