@@ -4,28 +4,28 @@ import gtk
 import gobject
 import gettext
 import tryton.common as common
-import tryton.rpc as rpc
-from tryton.exceptions import TrytonServerError
 import types
 from tryton.config import TRYTON_ICON
 import csv
 import tempfile
 import os
+from tryton.common import RPCExecute, RPCException
+from tryton.gui.window.nomodal import NoModal
 
 _ = gettext.gettext
 
 
-class WinExport(object):
+class WinExport(NoModal):
     "Window export"
 
     def __init__(self, model, ids, context=None):
-        self.parent = common.get_toplevel_window()
-        self.dialog = gtk.Dialog(
-                title= _("Export to CSV"),
-                parent=self.parent,
-                flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT
-                | gtk.WIN_POS_CENTER_ON_PARENT)
+        super(WinExport, self).__init__()
+
+        self.dialog = gtk.Dialog(title=_("Export to CSV"), parent=self.parent,
+            flags=gtk.DIALOG_DESTROY_WITH_PARENT)
+        self.dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
         self.dialog.set_icon(TRYTON_ICON)
+        self.dialog.connect('response', self.response)
 
         vbox = gtk.VBox()
         frame_predef_exports = gtk.Frame()
@@ -67,7 +67,7 @@ class WinExport(object):
         img_button = gtk.Image()
         img_button.set_from_stock('tryton-list-add', gtk.ICON_SIZE_BUTTON)
         button_select.set_image(img_button)
-        button_select.connect_after('clicked',  self.sig_sel)
+        button_select.connect_after('clicked', self.sig_sel)
         vbox_buttons.pack_start(button_select, False, False, 0)
 
         button_unselect = gtk.Button(_("_Remove"), stock=None,
@@ -76,7 +76,7 @@ class WinExport(object):
         img_button = gtk.Image()
         img_button.set_from_stock('tryton-list-remove', gtk.ICON_SIZE_BUTTON)
         button_unselect.set_image(img_button)
-        button_unselect.connect_after('clicked',  self.sig_unsel)
+        button_unselect.connect_after('clicked', self.sig_unsel)
         vbox_buttons.pack_start(button_unselect, False, False, 0)
 
         button_unselect_all = gtk.Button(_("Clear"), stock=None,
@@ -85,7 +85,7 @@ class WinExport(object):
         img_button = gtk.Image()
         img_button.set_from_stock('tryton-clear', gtk.ICON_SIZE_BUTTON)
         button_unselect_all.set_image(img_button)
-        button_unselect_all.connect_after('clicked',  self.sig_unsel_all)
+        button_unselect_all.connect_after('clicked', self.sig_unsel_all)
         vbox_buttons.pack_start(button_unselect_all, False, False, 0)
 
         hseparator_buttons = gtk.HSeparator()
@@ -97,7 +97,7 @@ class WinExport(object):
         img_button = gtk.Image()
         img_button.set_from_stock('tryton-save', gtk.ICON_SIZE_BUTTON)
         button_save_export.set_image(img_button)
-        button_save_export.connect_after('clicked',  self.add_predef)
+        button_save_export.connect_after('clicked', self.add_predef)
         vbox_buttons.pack_start(button_save_export, False, False, 0)
 
         button_del_export = gtk.Button(_("Delete Export"), stock=None,
@@ -106,7 +106,7 @@ class WinExport(object):
         img_button = gtk.Image()
         img_button.set_from_stock('tryton-delete', gtk.ICON_SIZE_BUTTON)
         button_del_export.set_image(img_button)
-        button_del_export.connect_after('clicked',  self.remove_predef)
+        button_del_export.connect_after('clicked', self.remove_predef)
         vbox_buttons.pack_start(button_del_export, False, False, 0)
 
         frame_export = gtk.Frame()
@@ -156,7 +156,6 @@ class WinExport(object):
         button_ok.set_flags(gtk.CAN_DEFAULT)
 
         self.dialog.vbox.pack_start(vbox)
-        self.dialog.show_all()
 
         self.ids = ids
         self.model = model
@@ -192,8 +191,6 @@ class WinExport(object):
 
         self.view1.set_model(self.model1)
         self.view2.set_model(self.model2)
-        self.view1.show_all()
-        self.view2.show_all()
 
         self.wid_action = combo_saveas
         self.wid_write_field_names = checkbox_add_field_names
@@ -207,13 +204,20 @@ class WinExport(object):
 
         self.pref_export.connect("row-activated", self.sel_predef)
 
-        # Fill the predefined export tree view and show everything
+        # Fill the predefined export tree view
         self.predef_model = gtk.ListStore(
                 gobject.TYPE_INT,
                 gobject.TYPE_PYOBJECT,
                 gobject.TYPE_STRING)
         self.fill_predefwin()
-        self.pref_export.show_all()
+
+        sensible_allocation = self.sensible_widget.get_allocation()
+        self.dialog.set_default_size(int(sensible_allocation.width * 0.9),
+            int(sensible_allocation.height * 0.9))
+        self.dialog.show_all()
+        common.center_window(self.dialog, self.parent, self.sensible_widget)
+
+        self.register()
 
     def model_populate(self, fields, parent_node=None, prefix_field='',
             prefix_name=''):
@@ -236,11 +240,10 @@ class WinExport(object):
                 self.model1.insert(node, 0, [None, '', 'white'])
 
     def _get_fields(self, model):
-        args = ('model', model, 'fields_get', None, rpc.CONTEXT)
         try:
-            return rpc.execute(*args)
-        except TrytonServerError, exception:
-            return common.process_exception(exception, *args)
+            return RPCExecute('model', model, 'fields_get', None)
+        except RPCException:
+            return ''
 
     def on_row_expanded(self, treeview, iter, path):
         child = self.model1.iter_children(iter)
@@ -274,30 +277,21 @@ class WinExport(object):
         self.model2.clear()
 
     def fill_predefwin(self):
-        args = ('model', 'ir.export', 'search',
-                [('resource', '=', self.model)], 0, None, None, rpc.CONTEXT)
         try:
-            export_ids = rpc.execute(*args)
-        except TrytonServerError, exception:
-            export_ids = common.process_exception(exception, *args)
-            if not export_ids:
-                return
-        args = ('model', 'ir.export', 'read', export_ids, None, rpc.CONTEXT)
+            export_ids = RPCExecute('model', 'ir.export', 'search',
+                [('resource', '=', self.model)], 0, None, None)
+        except RPCException:
+            return
         try:
-            exports = rpc.execute(*args)
-        except TrytonServerError, exception:
-            exports = common.process_exception(exception, *args)
-            if not exports:
-                return
-        args = ('model', 'ir.export.line', 'read',
-                sum((x['export_fields'] for x in exports), []), None,
-                rpc.CONTEXT)
+            exports = RPCExecute('model', 'ir.export', 'read', export_ids,
+                None)
+        except RPCException:
+            return
         try:
-            lines = rpc.execute(*args)
-        except TrytonServerError, exception:
-            lines = common.process_exception(exception, *args)
-            if not lines:
-                return
+            lines = RPCExecute('model', 'ir.export.line', 'read',
+                sum((x['export_fields'] for x in exports), []), None)
+        except RPCException:
+            return
         id2lines = {}
         for line in lines:
             id2lines.setdefault(line['export'], []).append(line)
@@ -318,19 +312,16 @@ class WinExport(object):
             field_name = self.model2.get_value(iter, 1)
             fields.append(field_name)
             iter = self.model2.iter_next(iter)
-        args = ('model', 'ir.export', 'create', {
-            'name': name,
-            'resource': self.model,
-            'export_fields': [('create', {
-                'name': x,
-                }) for x in fields],
-            }, rpc.CONTEXT)
         try:
-            new_id = rpc.execute(*args)
-        except TrytonServerError, exception:
-            new_id = common.process_exception(exception, *args)
-            if not new_id:
-                return
+            new_id, = RPCExecute('model', 'ir.export', 'create', [{
+                    'name': name,
+                    'resource': self.model,
+                    'export_fields': [('create', [{
+                                        'name': x,
+                                        } for x in fields])],
+                    }])
+        except RPCException:
+            return
         self.predef_model.append((
             new_id,
             fields,
@@ -345,12 +336,10 @@ class WinExport(object):
         if not i:
             return None
         export_id = model.get_value(i, 0)
-        args = ('model', 'ir.export', 'delete', export_id, rpc.CONTEXT)
         try:
-            rpc.execute(*args)
-        except TrytonServerError, exception:
-            if not common.process_exception(exception, *args):
-                return
+            RPCExecute('model', 'ir.export', 'delete', [export_id])
+        except RPCException:
+            return
         for i in range(len(self.predef_model)):
             if self.predef_model[i][0] == export_id:
                 del self.predef_model[i]
@@ -379,9 +368,18 @@ class WinExport(object):
                 continue
             self.model2.append((self.fields_data[field]['string'], field))
 
-    def run(self):
-        button = self.dialog.run()
-        if button == gtk.RESPONSE_OK:
+    def destroy(self):
+        super(WinExport, self).destroy()
+        self.dialog.destroy()
+
+    def show(self):
+        self.dialog.show()
+
+    def hide(self):
+        self.dialog.hide()
+
+    def response(self, dialog, response):
+        if response == gtk.RESPONSE_OK:
             fields = []
             fields2 = []
             iter = self.model2.get_iter_root()
@@ -390,8 +388,7 @@ class WinExport(object):
                 fields2.append(self.model2.get_value(iter, 0))
                 iter = self.model2.iter_next(iter)
             action = self.wid_action.get_active()
-            self.parent.present()
-            self.dialog.destroy()
+            self.destroy()
             result = self.datas_read(self.ids, self.model, fields,
                     context=self.context)
 
@@ -409,8 +406,7 @@ class WinExport(object):
                             self.wid_write_field_names.get_active())
             return True
         else:
-            self.parent.present()
-            self.dialog.destroy()
+            self.destroy()
             return False
 
     def export_csv(self, fname, fields, result, write_title=False, popup=True):
@@ -422,8 +418,8 @@ class WinExport(object):
             for data in result:
                 row = []
                 for val in data:
-                    if type(val) == types.StringType:
-                        row.append(val.replace('\n',' ').replace('\t',' '))
+                    if isinstance(type(val), types.StringType):
+                        row.append(val.replace('\n', ' ').replace('\t', ' '))
                     else:
                         row.append(val)
                 writer.writerow(row)
@@ -435,19 +431,14 @@ class WinExport(object):
                     common.message(_('%d records saved!') % len(result))
             return True
         except IOError, exception:
-            common.warning(_("Operation failed!\nError message:\n%s") \
-                     % (exception.faultCode,), _('Error'))
+            common.warning(_("Operation failed!\nError message:\n%s")
+                % (exception.faultCode,), _('Error'))
             return False
 
     def datas_read(self, ids, model, fields, context=None):
-        if context is None:
-            context = {}
-        ctx = context.copy()
-        ctx.update(rpc.CONTEXT)
         try:
-            datas = rpc.execute('model', model,
-                    'export_data', ids, fields, ctx)
-        except TrytonServerError, exception:
-            common.process_exception(exception)
+            datas = RPCExecute('model', model, 'export_data', ids, fields,
+                context=context)
+        except RPCException:
             return []
         return datas
